class GrantType:
    AUTH_CODE = 'authorisation_code'
    IMPLICIT = 'implicit';
    USER_CREDENTIAL = 'password'
    CLIENT_CREDENTIALS = 'client_credentials'
    TOKEN_REFRESH = 'refresh_token'
    JWT_BEARER = 'urn:ietf:params:oauth:grant-type:jwt-bearer'